use encrypted_filesystem_storage::*;

/// Must use the same nonce value for encryption and decryption -> store it with the header
#[test]
fn encryption_decryption_round() {
    use ring::aead::{Aad, BoundKey, Nonce, NonceSequence, OpeningKey, SealingKey, UnboundKey};

    let plaintext = util::random::<[u8; 128]>();
    let mut random_data = Vec::with_capacity(1024);
    random_data.extend(plaintext);

    let key_material: [u8; 32] = util::random();
    //let nonce = util::NonceCounter::random_lower_half();
    let nonce = util::NonceCounter::one();

    let algorithm = &ring::aead::CHACHA20_POLY1305;
    println!("{algorithm:?}");

    //let nonce = Nonce::try_assume_unique_for_key(&nonce).expect("unable to construct Nonce");
    let unbound_key =
        UnboundKey::new(algorithm, &key_material).expect("unable to construct UnboundKey 1");
    let mut key = SealingKey::new(unbound_key, nonce);

    // additional authenticated data (AAD), if any. This is authenticated but not encrypted. The type A could be a byte slice &[u8], a byte array [u8; N] for some constant N, Vec<u8>, etc. If there is no AAD then use Aad::empty()
    let aad = Aad::empty();
    let _encrypted: () = key
        .seal_in_place_append_tag(aad, &mut random_data)
        .expect("unable to encrypt");

    println!("key {key:?}");
    assert_ne!(&plaintext, &*random_data);

    //let nonce = key.nonce_sequence;

    let unbound_key =
        UnboundKey::new(algorithm, &key_material).expect("unable to construct UnboundKey 2");
    let nonce = util::NonceCounter::one();
    let mut key = OpeningKey::new(unbound_key, nonce);

    let aad = Aad::empty();
    let decrypted = key
        .open_in_place(aad, &mut random_data)
        .expect("unabel to decrypt");

    assert_eq!(&plaintext, &decrypted);
    assert_eq!(&plaintext, &random_data[..128]);
}

#[test]
fn counter_advance() {
    use ring::aead::NonceSequence;

    let mut nonce = util::NonceCounter::one();

    let two = nonce.advance().expect("unable to advance 2 counter");
    assert_eq!(&[2u8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], two.as_ref());
    let three = nonce.advance().expect("unable to advance 3 counter");
    assert_eq!(&[3u8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], three.as_ref());
}
